"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const engines_1 = __importDefault(require("./engines"));
const fsRead = util_1.promisify(fs_1.default.readFile);
class Renderer {
    constructor(packageDir = path_1.default.resolve(process.cwd(), 'node_modules')) {
        this.packageDir = packageDir;
        this._engineCache = {};
    }
    render(file, data) {
        const ext = path_1.default.extname(file).slice(1);
        const engine = this._getEngine(ext);
        return engine(file, data);
    }
    _getEngine(ext) {
        // Load from cache
        if (ext && this._engineCache[ext])
            return this._engineCache[ext];
        // Engine npm package name
        const enginePkgName = engines_1.default[ext];
        if (!enginePkgName) {
            throw new Error(`Origami.Renderer: No engine is configured for ${ext} filetype.`);
        }
        let engine;
        if (typeof enginePkgName === 'string') {
            // Attempt to load the package
            try {
                engine = enginePkgName
                    ? require(path_1.default.resolve(this.packageDir, enginePkgName))
                    : false;
            }
            catch (e) {
                throw new Error(`Origami.Renderer: '${enginePkgName}' engine is not installed.`);
            }
        }
        // Wrap the engine in a CompileFunction syntax
        switch (ext) {
            case 'pug':
                return this._engineCache[ext] = async (template, data) => {
                    const markdown = require('marked');
                    markdown.setOptions({
                        breaks: true,
                        gfm: true
                    });
                    const options = {
                        filename: template,
                        filters: { markdown }
                    };
                    return engine.compile((await fsRead(template)).toString(), options)(data);
                };
            case 'hbs':
            case 'ejs':
                return this._engineCache[ext] = async (template, data) => engine.compile((await fsRead(template)).toString())(data);
            case 'scss':
            case 'sass':
                return this._engineCache[ext] = (styles, options) => fs_1.default.createReadStream(styles).pipe(engine(styles, options));
            default:
                return (text) => fs_1.default.createReadStream(text);
        }
    }
}
exports.default = Renderer;
