"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dot_object_1 = __importDefault(require("dot-object"));
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const _1 = require(".");
const Route_1 = __importDefault(require("./Route"));
const fsReadFile = util_1.promisify(fs_1.readFile);
const fsWriteFile = util_1.promisify(fs_1.writeFile);
const fsStat = util_1.promisify(fs_1.stat);
const CONFIG_FILE = () => path_1.default.resolve(process.env.CLI_CWD || './', '.origami');
var config;
(function (config_1) {
    /**
     * Converts origami_x_y envrionment variables to an object
     * @returns {Object} Object of origami environment variables
    */
    config_1.env = (obj = {}) => {
        const e = Object.entries(process.env)
            .filter(([key, value]) => /^origami_.*$/.test(key))
            .map(([key, value]) => [
            key
                // Replace _ with .
                .replace(/_/g, '.')
                // Remove preceding 'origami.'
                .split('.').slice(1).join('.'),
            value
        ])
            .forEach(([key, value]) => {
            dot_object_1.default.str(key, value, obj);
        });
        return obj;
    };
    /**
     * Attempt to load the .origami file at the current directory. Overwrites with any ENV variables
     * @returns {Origami.Config|Boolean} The Origami file as json, or false if it cannot be
     * found or loaded correctly
    */
    config_1.read = async (fileOrDirectory) => {
        // If there is no file provided, load the .origami file in the current dir
        let file = path_1.default.join(process.env.CLI_CWD || process.cwd(), '.origami');
        if (fileOrDirectory) {
            const stats = await fsStat(fileOrDirectory);
            if (stats.isFile())
                file = fileOrDirectory;
            // If a directory is passed, attempt to load the .origami file in the directory
            // If that's not a file return false
            if (stats.isDirectory()) {
                const joined = path_1.default.join(fileOrDirectory, '.origami');
                const fileStats = await fsStat(joined);
                if (fileStats.isFile()) {
                    process.env.CLI_CWD = fileOrDirectory;
                    process.chdir(fileOrDirectory);
                    file = path_1.default.join(process.cwd(), '.origami');
                }
                else
                    return false;
            }
        }
        let c = {};
        let fileString;
        try {
            fileString = (await fsReadFile(file)).toString();
        }
        catch (_a) {
            // No .origami file
            return false;
        }
        try {
            c = JSON.parse(fileString);
        }
        catch (e) {
            console.log('Error parsing .origami file'.red);
            throw e;
        }
        c = config_1.env(c);
        return c;
    };
    /**
     * Override/write the .origami file
     * @param file JSON config for Origami app to override
     */
    config_1.write = async (file) => {
        const TAB_SIZE = 4;
        return fsWriteFile(CONFIG_FILE(), JSON.stringify(file, null, TAB_SIZE));
    };
    config_1.validate = (config) => {
        try {
            // requireKeys([
            //     'store'
            // ], config);
        }
        catch (e) {
            return _1.error(new Error(`Origami: Missing '${e.key}' setting`));
        }
        // ------------------------------------------------------ Validate store
        // const store = `origami-store-${config.store.type}`;
        // try {
        //     require(path.resolve(process.cwd(), 'node_modules', store, 'package.json'));
        // } catch (e) {
        //     if (e.name === 'Error') {
        //         console.log(path.resolve(process.cwd(), 'node_modules', store, 'package.json'));
        //         return error(
        //                 new Error(
        //                     `Origami: Could not find store plugin '${store}'. Try running 'yarn install ${store}'`
        //                 )
        //             );
        //     }
        //     return error(e);
        // }
    };
    // Setup the plugins for the server
    config_1.setupPlugins = (config, server, context = process.cwd()) => Object.entries(config.plugins).map(([name, settings]) => server.plugin(name, settings, context));
    // Setup the apps for the server
    config_1.setupApps = (config, server) => Object.entries(config.apps).map(([name, settings]) => server.application(name, settings));
    // Setup the resources for the server API
    config_1.setupResources = (config, server, context = process.cwd()) => Object.entries(config.resources).map(([name, r]) => {
        // r is a string to the model
        if (typeof r === 'string') {
            const model = require(path_1.default.resolve(context, r));
            const auth = true;
            return server.resource(name, { model, auth });
            // r is a config object
        }
        else if (r instanceof Object) {
            const model = require(path_1.default.resolve(context, r.model));
            const auth = r.auth;
            return server.resource(name, { model, auth });
        }
    });
    // Setup the controllers for the server API
    config_1.setupControllers = (config, server, context = process.cwd()) => Object.entries(config.controllers).map(async ([_path, c]) => {
        let config = {
            prefix: ''
        };
        if (typeof c === 'string') {
            config.prefix = c;
        }
        else if (c instanceof Object) {
            config = Object.assign({}, config, c);
        }
        const route = new Route_1.default(config.prefix);
        await route.include(path_1.default.resolve(context, _path), config.prefix, true);
        server.useRouter(route);
    });
    // Load a directory of routes, models, config files, etc, and automatically
    // add everything into the server (EG: calling useRouter(), resource(), etc)
    const loadDirectory = (dir) => {
        // TODO:
        _1.error('Not implemented');
    };
})(config = exports.config || (exports.config = {}));
