import express from 'express';
export declare namespace Origami {
    interface Config {
        /** Settings for the overall project */
        'app': ConfigApp;
        /** Settings for the store/database */
        'store'?: ConfigStore;
        /** Settings for the server setup */
        'server': ConfigServer;
        /** Admin node module */
        'admin'?: boolean | string;
        /** Model/Controller resources to automatically create */
        'resources'?: {
            [name: string]: ConfigResource | string;
        };
        /** Plugins to integrate into Origami */
        'plugins'?: {
            [name: string]: boolean | object;
        };
        /** Add controllers by individual files or directories */
        'controllers'?: {
            [path: string]: ConfigController | string;
        };
        /** Applications to install into Origami */
        'apps'?: {
            [name: string]: boolean | object;
        };
    }
    interface ConfigApp {
        /** Name of the project */
        'name': string;
    }
    interface ConfigStore {
        /** Store/Database type to integrate with */
        'type': string;
        /** Store/Database hostname to connect with */
        'host': string;
        /** Store/Database port to connect with */
        'port': number;
        /** Store/Database db name to connect with */
        'database': string;
        /** Store/Database username to connect with */
        'username': string;
        /** Store/Database password to connect with */
        'password': string;
    }
    interface ConfigTheme {
        /** Theme name to run */
        'name'?: string;
        'path'?: string;
    }
    interface ConfigServer {
        /** Secret code to encrypt data and authentication tokens with */
        'secret': string;
        /** Port number to run the server on */
        'port': number;
        /** Server language */
        'ln': string;
        /** Static directories to serve */
        'static'?: string | string[];
    }
    interface ConfigResource {
        model: string;
        auth?: boolean | {
            [key in 'get' | 'head' | 'post' | 'put' | 'delete' | 'list']: boolean;
        };
    }
    interface ConfigController {
        /** Prefix all controllers in this path under a prefix */
        prefix?: string;
    }
    /**
     * Valid types of Origami modules to install via NPM
     * @example origami-theme-snow, origami-store-mongodb, origami-plugin-facebook
     */
    type ModuleType = 'theme' | 'store' | 'plugin' | 'admin';
    namespace Server {
        type Position = 'init' | 'pre-store' | 'store' | 'post-store' | 'pre-render' | 'render' | 'post-render' | 'pre-send';
        type URL = string | null | RegExp;
        type Method = 'GET' | 'HEAD' | 'POST' | 'PUT' | 'DELETE' | 'CONNECT' | 'OPTIONS' | 'PATCH' | 'USE';
        interface Config {
            /** Secret code to encrypt data and authentication tokens with */
            'secret': string;
            /** Port number to run the server on */
            'port': number;
            /** Server language */
            'ln': string;
        }
        interface RequestHandler {
            (req: Request, res: Response, next: express.NextFunction): any;
        }
        interface Request extends express.Request {
            jwt: {
                token: string;
                data: {
                    userId: string;
                    email: string;
                };
            };
            __initialPassword?: string;
        }
        interface Response extends express.Response {
            data?: {
                [key: string]: any;
            } | any[];
            body?: string;
            text?: string;
            responseCode?: string;
            error: string;
            isPage: boolean;
            pageType: string | undefined;
        }
        interface NextFunction extends express.NextFunction {
        }
        interface DataError extends Error {
            data: object;
        }
    }
    namespace Theme {
        interface Config {
            /** Theme name to run */
            name: string;
            paths?: {
                styles?: string;
                views?: string;
                content?: string;
            };
        }
    }
    namespace Store {
        const Store: {
            new (options: StoreOptions): Store;
        };
        interface Store {
            models: {
                [name: string]: Model;
            };
            connURI: string;
            connect(): Promise<any>;
            model(name: string, schema?: Schema): Model | void;
        }
        interface StoreOptions {
            username: string;
            password: string;
            host: string;
            port: number;
            database: string;
        }
        interface Schema {
            tree?: boolean;
            properties: {
                [key: string]: any;
            };
        }
        const Model: {
            new (name: string, schema: Origami.Store.Schema): Model;
        };
        interface Model {
            create(resource: Origami.Store.Resource): Origami.Store.Resource;
            find(query: object, opts?: object): Promise<Resource | Resource[] | null>;
        }
        interface Resource {
            id?: string;
            deletedAt?: Date | null;
            children?: Resource[];
            [key: string]: any;
        }
    }
    interface AppManifest {
        name: string;
        icon: {
            type: string;
            color: string;
        };
        sidemenu?: boolean;
        resources?: object[];
        pages: AppManifestPage[];
    }
    interface AppManifestPage {
        title: string;
        path: string;
        page: string;
        icon?: string;
        scripts?: string[];
    }
}
export interface PackageJson {
    'name'?: string;
    'dependencies'?: {
        [pkg: string]: string;
    };
    'devDependencies'?: {
        [pkg: string]: string;
    };
    'scripts'?: {
        [name: string]: string;
    };
}
